/* Program: LDBDBisw.prg
   Author:  Garry Barrett, Stephen G. Donald
            last major update July, 2013
   Purpose: Apply the Lorenz Dominance tests proposed in Barrett, Donald
   and Bhattacharya "Consistent Nonparametric Tests for Lorenz Dominance" (2013)
   JBES, forthcoming.
   
   The test is for the null that Distribution X Lorenz Dominates Distribution Y.
   That is, the test is for H0: LC_x(p) >= LC_y(p) for all p,
                        and H1: LC_x(p) <  LC_y(p) for some p.

   In the notation of Bhattacharya (2003), distribution 1 == x
                                           distribution 2 == y.

Note:
This program for the test of LD is composed of 5 procedures:
ldtests(y,x)                             @ Calculate test stats            @
ldboot3(teststats,x,y,brep)              @ Bootstrap simulation            @
EmpLor(x)                                @ Calculate the Emp LC            @
LorInteg(gp,LCd)                         @ Integrate the Pos Diff of 2 LCs @
LorInter(gp,LCx)                         @ Interpolates Emp LC ordinates   @



This version allows for weighted data - this is introduced via
 the construction of the empirical lorenz curve.

Assumes Independent Sampling (IS).

The data matrix has 2 colunms,
 1 = income or consumption measure
 2 = record weight.
     Inverse probability weights are assumed. These are rescaled to sum
              to 1.0 in the program


	Last change: GB 14/10/2013 1:45:32 PM
*/

@ Set the file reference for writing the output @;
new;
library pgraph;
graphset;

@   Set Globals @;

@ 1. Set the file reference for inputting the data @
output file=c:\work\Lorenz\a8409p.out reset;
print " X = 84y, Y = 09y, Inc";

@ col 1=xadj 2=yadj 3=wgt     @

load x[2895,3]=c:\work\Lorenz\Aus\dat\aus84p.raw;
load y[5009,3]=c:\work\Lorenz\Aus\dat\aus09p.raw;


x=x[.,2]~x[.,3];
y=y[.,2]~y[.,3];


@ 2. Set the following, final global @
brep=2000;     @ # of bootstrap repetitions to calculate the P-values @

@ Normalise the total weight for each income vector to 1.0 @
x[.,2]=x[.,2]/sumc(x[.,2]);
y[.,2]=y[.,2]/sumc(y[.,2]);

mnx=sumc(x[.,1].*x[.,2]);
mny=sumc(y[.,1].*y[.,2]);
sdx=sumc( (x[.,1].^2).*x[.,2]) - mnx^2;
sdx=sdx^0.5;

sdy=sumc( (y[.,1].^2).*y[.,2]) - mny^2;
sdy=sdy^0.5;

nx=rows(x);
ny=rows(y);


ds=datestr(0);
ts=timestr(0);

print "=====================================================================";
print "Barrett, Donald and Bhattacharya (2013) Tests of Lorenz Dominance";
print "Date " ds "   " ts ;
print "=====================================================================";
print "  ";

et = hsec;       /* record the time length of this run */
rndseed 1364;    /* Set the seed for the random number generator */

{t1,ks2,te}=ldtests(y,x);    /* construct the test statistic */


{p1,pks2,pe} = ldboot3(t1,ks2,te,x,y,brep);  /* Bootstrapped P-value for LD */


print "LD Bootstrap Methods - Independent Sampling";
print "No. of iterations = " brep;
print "Mean X, SD_x, n_x =" mnx~sdx~nx;
print "Mean Y, SD_y, n_y =" mny~sdy~ny;
print "  ";
print "KS          test stat = " ks2;
print "            pvalue    = " pks2;
print "  ";
print "CvM         test stat = " t1;
print "            pvalue    = " p1;
print " ";
print "KS equality test stat = " te;
print "            pvalue    = " pe;
print " ";



/*   */
ds=datestr(0);ts=timestr(0);
print "Date " ds "   " ts ;
et = hsec - et;
print "Time, in seconds" et/100;

output off;
end;

/*====================== Proc: ldtests(y,x) =========================*/

proc (3)=ldtests(y,x);

  local j,n,m,zx,zy, mnx, mny, pdx, pdy,LCx,LCy,lam,gp,gpc,
        gLCx, gLCy, LCd, indx, indy, kx, ky, pqx, pqy, Ud, U, ld1,ks,kse;

  n = rows(x);  m = rows(y);
  lam = sqrt(n*m/(n+m));
/* construct the empirical LC for x and y at each distinct population quantile */

{LCx} = EmpLor(x);
{LCy} = EmpLor(y);

/* Now construct LCs from the empirical LC defined over all empirical population
   proportions defined in either sample

   First - define the set of p, gp, for the comparisons.
         - the expanded LCs are denoted by gLCx and gLCy
*/

gp=unique(LCx[.,1]|LCy[.,1],1);
{gLCx}=LorInter(gp,LCx);
{gLCy}=LorInter(gp,LCy);

/* Take the difference between the gLCs and integrate the positive diff only */

LCd= (gLCy-gLCx);

{Ud} = LorInteg(gp,LCd);

 @ Bhattacharya test @
 U = lam*sumc(Ud);

 @ KS test    @
 ks  = lam*maxc(LCd);

 @ KS test for equality of LCs  @
 kse  = lam*maxc(abs(LCd));


@  xy(gp,gLCx~gLCy); @
@  xy(gp,LCd);    @

retp(U,ks,kse);
endp;

/*=============== Proc: ldboot3(testst,ksstat,x,y,brep) ===============*/
proc (3)=ldboot3(testst,ks3,kse,x,y,brep);

local n,m,xs,ys,j,i,ib,ux,uy,sqn,sqm,lam,prej,pval2,pval3,ksp3,kspe,
LCx,LCy,LCxs,LCys,gp,zgp,gLCx,gLCy,gLCxs,gLCys,LCd,LCds,LCdds,Ud,
Us,tval3,tvale;

  n = rows(x);  m=rows(y);
  lam=sqrt(n*m/(n+m));

prej=zeros(3,1);


{LCx} = EmpLor(x);
{LCy} = EmpLor(y);

  for ib (1,brep,1);

/* initialize the vector corresponding to the resampled y */
   ys=zeros(m,2);

/* resample the x */
   uy=floor(m*rndu(m,1)+ones(m,1));
   for i (1,m,1);
     ys[i,1]=y[uy[i,1],1];
     ys[i,2]=y[uy[i,1],2];
   endfor;

/* initialize the vector corresponding to the resampled x */
   xs=zeros(n,2);

/*  resample the x */
   ux=floor(n*rndu(n,1)+ones(n,1));
   for i (1,n,1);
     xs[i,1]=x[ux[i,1],1];
     xs[i,2]=x[ux[i,1],2];
   endfor;

@ Normalise the total weight to 1.0 @
ys[.,2]=ys[.,2]/sumc(ys[.,2]);
xs[.,2]=xs[.,2]/sumc(xs[.,2]);

/* Construct the resampled empirical LCs */

{LCxs}=EmpLor(xs);
{LCys}=EmpLor(ys);

gp=unique(LCx[.,1]|LCy[.,1]|LCxs[.,1]|LCys[.,1],1);

{gLCx } = LorInter(gp,LCx);
{gLCy } = LorInter(gp,LCy);
{gLCxs} = LorInter(gp,LCxs);
{gLCys} = LorInter(gp,LCys);

LCd  = gLCy - gLCx;
LCds = gLCys- gLCxs;
LCdds= LCds -LCd;

/* Integrate over the positive difference between the 4 LCs */

{Ud}=LorInteg(gp,LCdds);

@ PID test stat @
   Us = lam*sumc(Ud);


@ KS test stat     @
   tval3=lam*maxc(LCdds);

@ KS test stat for LC equality    @
   tvale=lam*maxc(abs(LCdds));


    if Us > testst;    prej[1] = prej[1]+1;   endif;
    if tval3 > ks3;    prej[2] = prej[2]+1;   endif;
    if tvale > kse;    prej[3] = prej[3]+1;   endif;

  endfor;

  pval3 = prej[1]/brep;
  ksp3  = prej[2]/brep;
  kspe  = prej[3]/brep;

 retp(pval3,ksp3,kspe);

endp;

 /*====================== Proc: EmpLor(x) =========================*/

proc (1)=EmpLor(x);

  /* input  vector x - col.1=income, col.2= pop. proportion */
  /* output vector y - col.1=cum pop share, col.2=cum income share */
  local j,n,zx, mnx,pdx,LCx;

  /* construct the empirical LC for x - LC defined at each distinct population quantile */

n = rows(x);
zx=unique(x[.,1],1);
pdx=rows(zx);
mnx=meanc(x);
LCx=zeros(pdx+1,2);

@ Construct Empirical LCs for X and Y separately @

  for j (1,pdx,1);    /* construct the empirical LC for X  - start at 0 */
      LCx[j+1,1] = sumc((x[.,1] .le zx[j]).*x[.,2])/sumc(x[.,2]);
      LCx[j+1,2] = sumc((x[.,1] .le zx[j]).*x[.,1].*x[.,2])/sumc(x[.,1].*x[.,2]);
  endfor;

retp(LCx);
endp;

 /*====================== Proc: LorInter(gp,LCx) =========================*/

proc (1)=LorInter(gp,LCx);
  local j,n,zgp,gLCx,indx,kx,pqx;


@ Construct LCs from the empirical LC defined for a set of population proportions @
@ using linear interpolation @

@ Input - gp = pre-specificed population proportions @
@       - LCx = Empirical Lorenz curve ... col 1 == pop proportion @
@                                          col 2 == Lor ordinate   @
@                                                                  @
@ Output - gLCx = Lorenz ordinate corresponding to gp              @

n = rows(LCx);
zgp=rows(gp);
gLCx=zeros(zgp,1);

indx=seqa(1,1,n);

for j (2,zgp-1,1);

kx=0; pqx=0;
pqx=maxc((LCx[.,1].le gp[j]).*LCx[.,1]);
kx = maxc(indx.*(LCx[.,1].eq pqx));
if (kx eq (n));
    gLCx[j]=LCx[kx,2];
else;
    gLCx[j]=LCx[kx,2] + ((LCx[kx+1,2]-LCx[kx,2])/(LCx[kx+1,1]-LCx[kx,1]))*(gp[j]-LCx[kx,1]);
endif;

endfor;

gLCx[zgp]=1;

retp(gLCx);
endp;

 /*====================== Proc: LorInteg(gp,LCd) =========================*/

proc (1)=LorInteg(gp,LCd);

  local j,zgp,gpc,Ud;


zgp=rows(gp);
Ud=zeros(zgp,1);

for j (2,zgp-1,1);

/* Doing the integration for each segment of the LCs sequentially */

@ the area between the LCs for the first segment is a triangle @
if (j eq 2);
   if (LCd[j] gt 0);
       Ud[j] = gp[j]*LCd[j]/2;
   endif;
else;

@ now deal with the next series of segments which may a trapezium if there is no crossing on the interval @
@ or a triangle if there is an crossing on the interval - deal with each of the cases in turn             @

   if ((LCd[j] ge 0) and (LCd[j-1] ge 0));

@ this first branch deals with weak dominance over the interval @

       Ud[j] = (gp[j]-gp[j-1])*(LCd[j]+LCd[j-1])/2;


   elseif ((LCd[j] lt 0) and (LCd[j-1] gt 0));

@ this second branch deals with LCy crossing LCx from above           @
@ need to determine the population proportion where the LCs intersect @
@ then determine the area of the triangle                             @

   gpc= -(LCd[j-1])*(gp[j]-gp[j-1])/(LCd[j]-LCd[j-1]) ;

   Ud[j] =  (LCd[j-1])*gpc/2;


   elseif ((LCd[j] gt 0) and (LCd[j-1] lt 0));

@ this third branch deals with LCy crossing LCx from below            @
@ need to determine the population proportion where the LCs intersect @
@ then determine the area of the triangle                             @

   gpc= -(LCd[j-1])*(gp[j]-gp[j-1])/(LCd[j]-LCd[j-1]) ;

   Ud[j] =  (LCd[j])*(gp[j]-gp[j-1]- gpc)/2;

  endif;
endif;

endfor;

@ Now deal with the last segment of the LCs and the last piece of integration @


  if (LCd[zgp-1] gt 0);
      Ud[zgp]= LCd[zgp-1]*(1-gp[zgp-1])/2;
  endif;

retp(Ud);
endp;
